@@ -1739,11 +1739,16 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     prop = xsltGetNsProp(template, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop != NULL) {
         const xmlChar *URI;
+	xsltTemplatePtr cur;
 
-	if (ret->name != NULL) xmlFree(ret->name);
-	ret->name = NULL;
-	if (ret->nameURI != NULL) xmlFree(ret->nameURI);
-	ret->nameURI = NULL;
+	if (ret->name != NULL) {
+	    xmlFree(ret->name);
+	    ret->name = NULL;
+	}
+	if (ret->nameURI != NULL) {
+	    xmlFree(ret->nameURI);
+	    ret->nameURI = NULL;
+	}
 
 	URI = xsltGetQNameURI(template, &prop);
 	if (prop == NULL) {
@@ -1761,6 +1766,16 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 		ret->nameURI = xmlStrdup(URI);
 	    else
 		ret->nameURI = NULL;
+	    cur = ret->next;
+	    while (cur != NULL) {
+	        if (xmlStrEqual(cur->name, prop)) {
+		    xsltTransformError(NULL, style, template,
+		        "xsl:template: error duplicate name '%s'\n", prop);
+		    style->errors++;
+		    goto error;
+		}
+		cur = cur->next;
+	    }
 	}
     }
 
@@ -1904,7 +1919,8 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     } else if (IS_XSLT_NAME(cur, "namespace-alias")) {
 	    xsltNamespaceAlias(style, cur);
 	} else {
-            if ((style != NULL) && (style->doc->version != NULL) && (!strncmp(style->doc->version,"1.0",3))) {
+            if ((style != NULL) && (style->doc->version != NULL) &&
+	        (!strncmp((const char *) style->doc->version, "1.0", 3))) {
 	        xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: unknown %s element\n",
 			cur->name);
@@ -1958,7 +1974,8 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	return(NULL);
     }
     xsltParseStylesheetExcludePrefix(ret, cur);
-    xsltPrecomputeStylesheet(ret, cur);
+    if (!ret->nopreproc)
+	xsltPrecomputeStylesheet(ret, cur);
 
     if ((IS_XSLT_ELEM(cur)) && 
 	((IS_XSLT_NAME(cur, "stylesheet")) ||
